<?php
/**
 * CET Summer time method
 * 
 * @author Floris Luiten <www.florisluiten.nl>
 * @package CET
 * @subpackage summertime
 * @version $Id: cetsummertime (cetsummertime.php), v1.01 2011/02/19 $
 * @copyright Please contact author for copyright information, but
 *            in general these rules apply:
 *            
 *            - Want to use this code to learn how to write your own
 *              classes or teach others how to code? Go ahead! Please 
 *              give me credits in a proper way. Also, I'd love to hear
 *              your from you how you used this code. 
 *              
 *            - Want to use this code to generate a library for personal
 *              use? Please, go ahead! I'd love to hear from you how 
 *              this code helped you and where you have applied it.
 *              
 *            - Want to make money off this code? Please don't do that
 *              without consulting me first. If you really don't want
 *              to contact me for whatever reason, donate a FAIR amount
 *              to charity, in relation to how much money you make.
 */
/**
 * Datename of Monday
 * @var const
 */
define('DATENAME_MONDAY', 1);
/**
 * Datename of Tuesday
 * @var const
 */
define('DATENAME_TUESDAY', 2);
/**
 * Datename of Wednesday
 * @var const
 */
define('DATENAME_WEDNESDAY', 3);
/**
 * Datename of Thursday
 * @var const
 */
define('DATENAME_THURSDAY', 4);
/**
 * Datename of Friday
 * @var const
 */
define('DATENAME_FRIDAY', 5);
/**
 * Datename of Saturday
 * @var const
 */
define('DATENAME_SATURDAY', 6);
/**
 * Datename of Sunday
 * @var const
 */
define('DATENAME_SUNDAY', 7);
/**
 * Returns the day of the month of the first occurence of the dayName in
 * the specified year/month.
 * For example, running this function with the following parameters: 
 * DATENAME_MONDAY, 2000, 11 will return the first Monday in the month 
 * of november 2000. 
 *
 * @param integer The dayname (use const's DATENAME_MONDAY et cetera)
 * @param integer The year
 * @param integer The month
 * @author Floris Luiten <www.florisluiten.nl>
 */
function firstDaynameOfMonth($dayName, $year, $month) {
	$firstDayOfMonthDayName = date('N', mktime(0, 0, 0, $month, 1, $year)); 
	if ($firstDayOfMonthDayName == $dayName) {
		$firstMondayOfMonth = 1;
	} elseif ($firstDayOfMonthDayName > $dayName) {
		$firstMondayOfMonth = $dayName + 7 - $firstDayOfMonthDayName + 1;
	} elseif ($firstDayOfMonthDayName < $dayName) {
		$firstMondayOfMonth = $dayName - $firstDayOfMonthDayName + 1;
	}
	return $firstMondayOfMonth;
}

/**
 * Returns the day of the month of the last occurence of the dayName in
 * the specified year/month.
 * For example, running this function with the following parameters: 
 * DATENAME_MONDAY, 2000, 11 will return the last Monday in the month 
 * of november 2000. 
 *
 * @param integer The dayname (use const's DATENAME_MONDAY et cetera)
 * @param integer The year
 * @param integer The month
 */
function lastDaynameOfMonth($dayName, $year, $month) {
	$maxDaysOfMonth = date('t', mktime(0, 0, 0, $month, 1, $year));
	$lastDayOfMonthDayName = date('N', mktime(0, 0, 0, $month, $maxDaysOfMonth, $year));
	
	$lastDayOfMonth = -1;
	if ($lastDayOfMonthDayName == $dayName) {
		$lastDayOfMonth = $maxDaysOfMonth;
	} elseif ($lastDayOfMonthDayName > $dayName) {
		$lastDayOfMonth = $maxDaysOfMonth - $lastDayOfMonthDayName + $dayName;
	} elseif ($lastDayOfMonthDayName < $dayName) {
		$lastDayOfMonth = $maxDaysOfMonth - 7 - $lastDayOfMonthDayName + $dayName;
	}
	return $lastDayOfMonth;
}
/**
 * Finds out if the given datetime is in a CET summertime period 
 * according to the Dutch regulations
 * (http://www.knmi.nl/cms/content/24193/tijd_de_wettelijk_tijd_in_nederland)
 *
 * @param integer The year
 * @param integer The month
 * @param integer The day
 * @param integer The hour
 * @param integer The minute
 * @param integer The second
 * @return boolean True if the date is a summertime, false otherwise
 * @author Floris Luiten <www.florisluiten.nl>
 */
function CETisSummerTime($year, $month, $day, $hour, $minute, $second) {
	$theDate = mktime($hour, $minute, $second, $month, $day, $year);
	if ($theDate < mktime(0, 0, 0, 1, 5, 1916)) {
		return false;
	} elseif (
			($theDate >= mktime(0, 0, 0, 1, 5, 1916) AND $theDate <= mktime(23, 0, 0, 9, 30, 1916)) OR
			($theDate >= mktime(2, 0, 0, 4, 16, 1917) AND $theDate <= mktime(2, 0, 0, 9, 17, 1917)) OR 
			($theDate >= mktime(2, 0, 0, 3, 26, 1922) AND $theDate <= mktime(2, 0, 0, 10, 8, 1922)) OR
			($theDate >= mktime(2, 0, 0, 6, 1, 1923) AND $theDate <= mktime(2, 0, 0, 10, 7, 1923)) OR
			($theDate >= mktime(2, 0, 0, 3, 30, 1924) AND $theDate <= mktime(2, 0, 0, 10, 5, 1924)) OR
			($theDate >= mktime(2, 0, 0, 6, 5, 1925) AND $theDate <= mktime(2, 0, 0, 10, 4, 1925)) OR
			($theDate >= mktime(0, 0, 0, 5, 16, 1940) AND $theDate <= mktime(2, 0, 0, 11, 2, 1942)) OR
			($theDate >= mktime(2, 0, 0, 3, 29, 1943) AND $theDate <= mktime(2, 0, 0, 10, 4, 1943)) OR
			($theDate >= mktime(2, 0, 0, 4, 3, 1944) AND $theDate <= mktime(2, 0, 0, 10, 2, 1944)) OR
			($theDate >= mktime(2, 0, 0, 4, 2, 1945) AND $theDate <= mktime(2, 0, 0, 9, 16, 1945)) OR
			(CETisSummerTimeMethodOne($year, $month, $day, $hour, $minute, $second)) OR
			(CETisSummerTimeMethodTwo($year, $month, $day, $hour, $minute, $second)) OR
			(CETisSummerTimeMethodThree($year, $month, $day, $hour, $minute, $second)) OR
			(CETisSummerTimeMethodFour($year, $month, $day, $hour, $minute, $second)) OR
			(CETisSummerTimeMethodFive($year, $month, $day, $hour, $minute, $second))) {
		return true;
	}
	return false;
}

/**
 * Finds out if the given datetime is in a CET summertime period between
 * 1918 and 1921 according to these rules:
 * - Summertime starts at the first Monday of April
 * - Summertime ends at the last Monday of September
 *
 * @param integer The year
 * @param integer The month
 * @param integer The day
 * @param integer The hour
 * @param integer The minute
 * @param integer The second
 * @return boolean True if the date is a summertime, false otherwise
 * @author Floris Luiten <www.florisluiten.nl>
 */
function CETisSummerTimeMethodOne($year, $month, $day, $hour, $minute, $second) {
	if ($year >= 1918 AND $year <= 1921) {
		$startST = mktime(2, 0, 0, 4, firstDaynameOfMonth(DATENAME_MONDAY, $year, 4), $year);
		$endST = mktime(2, 0, 0, 9, lastDaynameOfMonth(DATENAME_MONDAY, $year, 9), $year);
		
		$theDate = mktime($hour, $minute, $second, $month, $day, $year);
		if ($theDate >= $startST AND $theDate <= $endST) {
			return true;
		}
	}
	return false;	
}

/**
 * Finds out if the given datetime is in a CET summertime period between
 * 1926 and 1939 according to these rules:
 * - Summertime starts at 15 May (22 May in 1932 and 1937)
 * - Summertime ends at the first Sunday at or past 2 October
 *
 * @param integer The year
 * @param integer The month
 * @param integer The day
 * @param integer The hour
 * @param integer The minute
 * @param integer The second
 * @return boolean True if the date is a summertime, false otherwise
 * @author Floris Luiten <www.florisluiten.nl>
 */
function CETisSummerTimeMethodTwo($year, $month, $day, $hour, $minute, $second) {
	if ($year >= 1926 AND $year <= 1939) {
		if ($year == 1932 OR $year == 1937) {
			$startST = mktime(2, 0, 0, 5, 22, $year);
		} else {
			$startST = mktime(2, 0, 0, 5, 15, $year);
		}
		$endSTSunday = firstDaynameOfMonth(DATENAME_SUNDAY, $year, 10);
		if ($endSTSunday < 2) {
			$endSTSunday += 7;
		}
		$endST = mktime(2, 0, 0, 10, $endSTSunday, $year);
		
		$theDate = mktime($hour, $minute, $second, $month, $day, $year);
		if ($theDate >= $startST AND $theDate <= $endST) {
			return true;
		}
	}
	return false;		
}

/**
 * Finds out if the given datetime is in a CET summertime period between
 * 1977 and 1980 according to these rules:
 * - Summertime starts at the first Sunday of April
 * - Summertime ends at the last Sunday at or before 1 October
 *
 * @param integer The year
 * @param integer The month
 * @param integer The day
 * @param integer The hour
 * @param integer The minute
 * @param integer The second
 * @return boolean True if the date is a summertime, false otherwise
 * @author Floris Luiten <www.florisluiten.nl>
 */
function CETisSummerTimeMethodThree($year, $month, $day, $hour, $minute, $second) {
	if ($year >= 1977 AND $year <= 1980) {
		$startST = mktime(2, 0, 0, 4, firstDaynameOfMonth(DATENAME_SUNDAY, $year, 4), $year);
		
		$endSTSunday = lastDaynameOfMonth(DATENAME_SUNDAY, $year, 9);
		if ($endSTSunday < 25) {
			$endSTSunday += 7;
		}
		$endST = mktime(2, 0, 0, 9, $endSTSunday, $year);
		
		$theDate = mktime($hour, $minute, $second, $month, $day, $year);
		if ($theDate >= $startST AND $theDate <= $endST) {
			return true;
		}
	}
	return false;		
}

/**
 * Finds out if the given datetime is in a CET summertime period between
 * 1981 and 1995 according to these rules:
 * - Summertime starts at the last Sunday of March
 * - Summertime ends at the last Sunday in September
 *
 * @param integer The year
 * @param integer The month
 * @param integer The day
 * @param integer The hour
 * @param integer The minute
 * @param integer The second
 * @return boolean True if the date is a summertime, false otherwise
 * @author Floris Luiten <www.florisluiten.nl>
 */
function CETisSummerTimeMethodFour($year, $month, $day, $hour, $minute, $second) {
	if ($year >= 1981 AND $year <= 1995) {
		$startST = mktime(2, 0, 0, 3, lastDaynameOfMonth(DATENAME_SUNDAY, $year, 3), $year);
		$endST = mktime(2, 0, 0, 9, lastDaynameOfMonth(DATENAME_SUNDAY, $year, 9), $year);
		
		$theDate = mktime($hour, $minute, $second, $month, $day, $year);
		if ($theDate >= $startST AND $theDate <= $endST) {
			return true;
		}
	}
	return false;		
}

/**
 * Finds out if the given datetime is in a CET summertime period between
 * 1981 and 1995 according to these rules:
 * - Summertime starts at the last Sunday of March
 * - Summertime ends at the last Sunday in October
 *
 * @param integer The year
 * @param integer The month
 * @param integer The day
 * @param integer The hour
 * @param integer The minute
 * @param integer The second
 * @return boolean True if the date is a summertime, false otherwise
 * @author Floris Luiten <www.florisluiten.nl>
 */
function CETisSummerTimeMethodFive($year, $month, $day, $hour, $minute, $second) {
	if ($year >= 1996) {
		$startST = mktime(2, 0, 0, 3, lastDaynameOfMonth(DATENAME_SUNDAY, $year, 3), $year);
		$endST = mktime(2, 0, 0, 10, lastDaynameOfMonth(DATENAME_SUNDAY, $year, 10), $year);
		
		$theDate = mktime($hour, $minute, $second, $month, $day, $year);
		if ($theDate >= $startST AND $theDate <= $endST) {
			return true;
		}
	}
	return false;		
}

$isInST = false;

for ($year = 1909; $year <= 2012; $year++) {
	for ($month = 1; $month <= 12; $month++) {
		for ($day = 1; $day <= date('t', mktime(0, 0, 0, $month, 1, $year)); $day++) {
			if (CETisSummerTime($year, $month, $day, 3, 0, 0) AND !$isInST) {
				echo date('d-m-Y', mktime(0, 0, 0, $month, $day, $year)) . ': Start of summertime<br>' . "\n"; 
				$isInST = true;
			} elseif (!CETisSummerTime($year, $month, $day, 23, 59, 59) AND $isInST) {
				echo date('d-m-Y', mktime(0, 0, 0, $month, $day, $year)) . ': End of summertime<br><br>' . "\n"; 
				$isInST = false;
			}
		}
	}
}

?>
